/*
 * FIFO.c
 *
 * Created: 17/08/2017 11:19:45
 *  Author: Luca
 */ 

 #include "FIFO.h"

 #define SIZE 100

 static char tx_buffer[SIZE], rx_buffer[SIZE];
 static int tx_head, rx_head;     
 static int tx_tail, rx_tail;

 void FIFO_init(void)
 {
	tx_head = tx_tail = 0;
	rx_head = rx_tail = 0;
 }

 int TX_FIFO_is_empty(void)
 {
	return tx_tail == tx_head;
 }

 int TX_FIFO_is_full(void)
 {
	return (tx_tail + 1) % SIZE == tx_head;
 }

 int TX_FIFO_put(char data)
 {
	if (TX_FIFO_is_full())
		return 0;
	tx_buffer[tx_tail] = data;
	tx_tail = (tx_tail + 1) % SIZE;
	return 1;
 }

 int TX_FIFO_get(char *data)
 {
	if (TX_FIFO_is_empty())
		return 0;
	*data = tx_buffer[tx_head];
	tx_head = (tx_head + 1) % SIZE;
	return 1;
 }
 
 int RX_FIFO_is_empty(void)
 {
	 return rx_tail == rx_head;
 }

 int RX_FIFO_is_full(void)
 {
	 return (rx_tail + 1) % SIZE == rx_head;
 }

 int RX_FIFO_put(char data)
 {
	 if (RX_FIFO_is_full())
		return 0;
	 rx_buffer[rx_tail] = data;
	 rx_tail = (rx_tail + 1) % SIZE;
	 return 1;
 }

 int RX_FIFO_get(char *data)
 {
	 if (RX_FIFO_is_empty())
		 return 0;
	 *data = rx_buffer[rx_head];
	 rx_head = (rx_head + 1) % SIZE;
	 return 1;
 }


