/**
 * \file
 *
 * \brief Empty user application template
 *
 */

/**
 * \mainpage User Application template doxygen documentation
 *
 * \par Empty user application template
 *
 * Bare minimum empty user application template
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to system_init()
 * -# "Insert application code here" comment
 *
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
/*
 * Support and FAQ: visit <a href="http://www.atmel.com/design-support/">Atmel Support</a>
 */
#include <samd21g18a.h>
#include "FIFO.h"

void clock_init(void);
void port_init(void);
void NVIC_init(void);
void USART_init(void);

/* blocking functions */
void USART_send_wait(char data);
char USART_receive_wait(void);
void USART_write_wait(const char *string);
void USART_writeln_wait(const char *string);
void USART_read_wait(char *string);

/* non blocking functions */
void USART_send(char data);
char USART_receive(void);
void USART_write(const char *string);
void USART_writeln(const char *string);
void USART_read(char *string);

void blink(void);

int main(void)
{
	clock_init();
	port_init();
	FIFO_init();
	USART_init();
	NVIC_init();

	for (int i = 4 ; i > 0 ; i--)
		USART_writeln("all work and no play makes jack a dull boy");   // write to tx buffer and return (don't wait for the USART to be ready)
	
	char buffer[100]; 

	while (1)
	{
		blink();                             // do something else
		USART_read(buffer);                  // read from rx buffer when needed (without waiting for data) TODO: implement timeout
		USART_write("received: ");
		USART_writeln(buffer);
	}
}

void blink(void)
{
	for (int l = 0 ; l < 10 ; l++)
	{
		PORT->Group[0].OUTTGL.reg = PORT_PA17;
		for (int i = 0 ; i < 1000000 ; i++)
			;
		PORT->Group[0].OUTTGL.reg = PORT_PA17;
		for (int i = 0 ; i < 1000000 ; i++)
			;
	}
}

void clock_init(void)
{
	SYSCTRL->OSC8M.bit.PRESC = 0;                          // no prescaler (is 8 on reset)
	SYSCTRL->OSC8M.reg |= 1 << SYSCTRL_OSC8M_ENABLE_Pos;   // enable source

	GCLK->GENDIV.bit.ID = 0x03;                            // select GCLK_GEN[3]
	GCLK->GENDIV.bit.DIV = 0;                              // no prescaler

	GCLK->GENCTRL.bit.ID = 0x03;                           // select GCLK_GEN[3]
	GCLK->GENCTRL.reg |= GCLK_GENCTRL_SRC_OSC8M;           // OSC8M source
	GCLK->GENCTRL.bit.GENEN = 1;                           // enable generator

	GCLK->CLKCTRL.reg = GCLK_CLKCTRL_ID_SERCOM5_CORE;      // SERCOM5 multiplexer GCLK_PERIPHERAL[n]
	GCLK->CLKCTRL.reg |= GCLK_CLKCTRL_GEN_GCLK3;           // select multiplexer source GCLK_GEN[3]
	GCLK->CLKCTRL.bit.CLKEN = 1;                           // enable generic clock

	PM->APBCSEL.bit.APBCDIV = 0;                           // no prescaler
	PM->APBCMASK.bit.SERCOM5_ = 1;                         // enable SERCOM5
}

void port_init(void)
{
	/* assign alternate function to pin PB22 and PB23 */
	PORT->Group[1].PINCFG[22].bit.PMUXEN = 1;  
	PORT->Group[1].PINCFG[23].bit.PMUXEN = 1;
	PORT->Group[1].PMUX[11].bit.PMUXE = 0x03;  // pin PB22 is assigned to SERCOM5 PAD2 (TX)
	PORT->Group[1].PMUX[11].bit.PMUXO = 0x03;  // pin PB23 is assigned to SERCOM5 PAD3 (RX)

	/* pin PA17 output (Arduino Pin 13 --> on board user led) */
	PORT->Group[0].DIR.reg |= PORT_PA17;
	PORT->Group[0].OUTCLR.reg = PORT_PA17;
}

void USART_init(void)
{
	// configure USART 8N1 
	SERCOM5->USART.CTRLA.bit.MODE = 0x1;                               // USART internal clock
	SERCOM5->USART.CTRLA.bit.CMODE = 0;                                // asynchronous mode
	SERCOM5->USART.CTRLA.bit.RXPO = 0x3;                               // RX on PAD3
	SERCOM5->USART.CTRLA.bit.TXPO = 0x1;                               // TX on PAD2
	SERCOM5->USART.CTRLA.bit.DORD = 1;                                 // data order: LSB first
	SERCOM5->USART.CTRLB.bit.CHSIZE = 0x0;                             // 8-bit character
	SERCOM5->USART.CTRLA.bit.FORM = 0;                                 // no parity bit
	SERCOM5->USART.CTRLB.bit.SBMODE = 0;                               // 1 stop bit 
	SERCOM5->USART.BAUD.reg = 50437;                                   // 115200 baud rate 
	SERCOM5->USART.INTENSET.bit.RXC = 1;                               // enable receive complete interrupt
	SERCOM5->USART.INTENSET.bit.DRE = 1;                               // enable data register empty interrupt
	SERCOM5->USART.CTRLB.bit.RXEN = 1;                                 // enable receiver
	SERCOM5->USART.CTRLB.bit.TXEN = 1;                                 // enable transmitter
	SERCOM5->USART.CTRLA.reg |= 1 << SERCOM_USART_CTRLA_ENABLE_Pos;    // enable USART
}

void NVIC_init(void)
{
	NVIC_EnableIRQ(SERCOM5_IRQn);       // enable SERCOM0 interrupt request
	NVIC_SetPriority(SERCOM5_IRQn,0);   // highest priority
}

void USART_send_wait(char data)
{
	while (!SERCOM5->USART.INTFLAG.bit.DRE)
		;
	SERCOM5->USART.DATA.reg = data;
}

char USART_receive_wait(void)
{
	while (!SERCOM5->USART.INTFLAG.bit.RXC)
		;
	return SERCOM5->USART.DATA.reg;
}

void USART_write_wait(const char *string)
{
	while (*string)
		USART_send_wait(*string++);
}
void USART_writeln_wait(const char *string)
{
	USART_write_wait(string);
	USART_write_wait("\r\n");
}

void USART_read_wait(char *string)
{
	while ((*string = USART_receive_wait()) != '\n')
		string++;
	*string = '\0';
}

void USART_send(char data)
{
	while (!TX_FIFO_put(data))
		;
	if (!(SERCOM5->USART.INTENSET.reg & SERCOM_USART_INTENSET_DRE))     // data put in tx buffer: re-enable DRE interrupt if disabled
		SERCOM5->USART.INTENSET.reg = SERCOM_USART_INTENSET_DRE;
}

char USART_receive(void)
{
	char data;
	while (!RX_FIFO_get(&data))
		;
	if (!(SERCOM5->USART.INTENSET.reg & SERCOM_USART_INTENSET_RXC))     // data received from rx buffer: re-enable RXC interrupt if disabled
		SERCOM5->USART.INTENSET.reg = SERCOM_USART_INTENSET_RXC;
	return data;
}

void USART_write(const char *string)
{
	while (*string)
		USART_send(*string++);
}

void USART_writeln(const char *string)
{
	USART_write(string);
	USART_write("\r\n");
}

void USART_read(char *string)
{
	while ((*string = USART_receive()) != '\n')
		string++;
	*string = '\0';
}

void SERCOM5_Handler(void)
{
	char data;
	if (SERCOM5->USART.INTFLAG.reg & SERCOM_USART_INTFLAG_DRE && SERCOM5->USART.INTENSET.reg & SERCOM_USART_INTENSET_DRE)  // if interrupt flag is set AND interrupt is enabled
	{
		if (!TX_FIFO_get(&data))   // if tx buffer is empty disable DRE interrupt
			SERCOM5->USART.INTENCLR.reg = SERCOM_USART_INTENCLR_DRE;
		else
			SERCOM5->USART.DATA.reg = data;		
	}
	if (SERCOM5->USART.INTFLAG.reg & SERCOM_USART_INTFLAG_RXC && SERCOM5->USART.INTENSET.reg & SERCOM_USART_INTENSET_RXC)  // if interrupt flag is set AND interrupt is enabled
	{
		data = SERCOM5->USART.DATA.reg;
		if (!RX_FIFO_put(data))   // if rx buffer is full disable RXC interrupt
			SERCOM5->USART.INTENCLR.reg = SERCOM_USART_INTENCLR_RXC;
	}
}

